import itertools

def expand(expression):
	"""Expand a string representing a node range into the individual nodes
	The node range string can contain multiple groups:
		>>> nodeset.expand('c[13-14]-[1-2]')
		['c13-1', 'c13-2', 'c14-1', 'c14-2']
	"""
	
	if expression.find("-")==-1 and expression.find("[")==-1:
		return [expression];
	
	groups = _range_locations(expression)
	fields = []
	last = -1
	for s, e in groups:
		if s > last:
			fields.append([expression[last+1:s]])
		fields.append(_expand_range(expression[s+1:e]))
		last = e
	if last < len(expression) - 1:
		fields.append(expression[last+1], len(expression))
	nodes = _product(fields)
	return [''.join(n) for n in nodes]


def _range_locations(expression):
	"""Return an array with the positions of all node groups"""
	start = []
	end = []
	for pos, char in enumerate(expression):
		if char == '[':
			start.append(pos)
		elif char == ']':
			end.append(pos)
	return zip(start, end)


def _expand_range(value):
	"""Expand a given node range like: '1-10,17,19'"""
	ranges = value.split(',')
	nodes = []
	for r in ranges:
		if '-' in r:
			start, end = r.split('-')
			nodes += [str(i) for i in range(int(start), int(end) + 1)]
		else:
			nodes.append(r.strip())
	return nodes


def _replace(txt, start, end, repltxt):
	"""Replaces the part of txt between index start and end with repltxt"""
	return txt[:start] + repltxt + txt[end+1:]


def _product(lists):
	"""Return a list with the cartesian product of the given lists"""
	return list(itertools.product(*lists))
