from collections import namedtuple
from anomaly.tools import nodeset
from anomaly import utils
import mysql.connector
import time
import datetime

Job = namedtuple('Job', 'id priority account user nodes_alloc nodelist cpus_req cpus_alloc '
'mem_req id_qos partition time_start time_end timelimit time_submit time_eligible '
'time_suspended time_elapsed start_week_day start_hour state')

def jobs_mysql(started_before=None, started_after=None, end=None, min_etime=0):
	"""Returns a list of jobs which meet the following criteria

	If no end time is given then currently running jobs are also returned.

	Parameters:
		- started_before: jobs that started before or exactly at this timestamp
		- started_after: jobs that started after or exactly at this timestamp
		- end: jobs that ended before this timestamp
		- min_etime: only return jobs that lasted more than min_etime seconds
	"""
	
	fields = ['ft2_job_table.id_job',
			'ft2_job_table.priority',
			'ft2_job_table.account',
			'ft2_assoc_table.user',
			'ft2_job_table.nodes_alloc',
			'ft2_job_table.nodelist',
			'ft2_job_table.cpus_req',
			'ft2_job_table.cpus_alloc',
			'ft2_job_table.mem_req',
			'ft2_job_table.id_qos',
			'ft2_job_table.partition',
			'ft2_job_table.time_start',
			'ft2_job_table.time_end',
			'ft2_job_table.timelimit',
			'ft2_job_table.time_submit',
			'ft2_job_table.time_eligible',
			'ft2_job_table.time_suspended',
			'ft2_job_table.state'
			]
	
	connection = mysql.connector.connect(host='<host addr>', user='<user>', password='<password>', database='slurm_acct_db')

	conditions = []
	if started_before:
		conditions.append('ft2_job_table.time_start <= {}'.format(started_before))
	if started_after:
		conditions.append('ft2_job_table.time_start >= {}'.format(started_after))
	if end:
		conditions.append('ft2_job_table.time_end <= {}'.format(end))
	if min_etime:
		conditions.append('ft2_job_table.time_end - ft2_job_table.time_start >= {}'.format(min_etime))

	where_conditions = ' and '.join(conditions)
	select_fields = ', '.join(fields)
	query = ('SELECT {select_fields} '
			'FROM ft2_job_table join ft2_assoc_table '
				'on (ft2_job_table.id_assoc = ft2_assoc_table.id_assoc) '
			'WHERE {where_conditions}'
			.format(select_fields = select_fields, where_conditions = where_conditions))

	print('Executing query:\n{}\n\n'.format(query))

	cursor = connection.cursor()
	now = int(time.time())
	cursor.execute(query)
	jobs = cursor.fetchall()

	ret = []
	for j in jobs:
		et = j[12] - j[11] if j[12] > 0 else now - j[11] # Elapsed time
		stdate = utils.to_date(j[11]) # Start time in date format
		split = stdate.split(" ")
		shour = split[1].split(":")[0] # Start hour
		split = split[0].split("-")
		swd = datetime.datetime(int(split[0]), int(split[1]), int(split[2])).weekday() # Start week day
		swd = utils.int_week_day_to_str(swd)

		ret.append(Job(
			id=j[0], priority=j[1], account=j[2], user=j[3], nodes_alloc=j[4], nodelist=j[5],
			cpus_req=j[6], cpus_alloc=j[7], mem_req=j[8], id_qos=j[9], partition=j[10], time_start=j[11],
			time_end=j[12], timelimit=j[13], time_submit=j[14], time_eligible=j[15], time_suspended=j[16],
			time_elapsed=et, start_week_day = swd, start_hour = shour, state=j[len(j)-1]
		))

	return ret
