import datetime
import time
from anomaly.tools import nodeset


def to_ts(day, month, year, hour, minutes, seconds=0):
    """Convert the given date to a timestamp value"""
    return int(time.mktime(datetime.datetime(
        year, month, day, hour, minutes, seconds).timetuple()))


def to_date(timestamp):
    """Converts a timestamp to a human-readable date"""
    return datetime.datetime.fromtimestamp(timestamp).strftime('%Y-%m-%d %H:%M:%S')

def int_state_to_str(int_state):
	"""Translates an integer state to its str meaning"""
	if int_state == 1 : return 'RUNNING'
	elif int_state == 3 : return 'COMPLETED'
	elif int_state == 4 : return 'CANCELLED'
	elif int_state == 5 : return 'FAILED'
	elif int_state == 6 : return 'TIMEOUT'
	else : return 'UNKNOWN'

def int_week_day_to_str(int_week_day):
	"""Translates an integer week day to its str meaning"""
	if int_week_day == 0 : return "monday"
	elif int_week_day == 1 : return "tuesday"
	elif int_week_day == 2 : return "wednesday"
	elif int_week_day == 3 : return "thursday"
	elif int_week_day == 4 : return "friday"
	elif int_week_day == 5 : return "saturday"
	elif int_week_day == 6 : return "sunday"
	else : return "unknown"

def split_nodes_to_list(split_nodes):
	"""Reads the split_nodes file and returns a list with its values
	     split_nodes file can be generated with: 'sinfo -Nr --hide'
	"""
	l = []
	fin = open(split_nodes,'r')
	
	line = fin.readline()[:-1]
	linelen = len(line)
	while linelen > 0:
		auxlist = nodeset.expand(line)
		l = l + auxlist
		line = fin.readline()[:-1]
		linelen = len(line)

	fin.close()
	return l


def training_csv_dump(filename, jobs, append=False, skip_header=False):
	"""Dump the received jobs to a training csv file.
		filename: Name of the file where data is going to be exported
		jobs: List of jobs obtained through the slurm.jobs_mysql() method
		append: True means the file will be written after its end.
			False means the file will be overwritten
		skip-header: True means header will be printed as first write
			False means no header will be printed
	"""
	fout = open(filename,'a' if append else 'w')
	if not skip_header:
		fout.write("id_job, priority, account, user, nodes_alloc, nodelist, cpus_req, "
			"cpus_alloc, mem_req, id_qos, partition, time_start, time_end, timelimit, " 
			"time_submit, time_eligible, time_suspended, time_elapsed, start_week_day, "
			"start_hour, state\n")
	for j in jobs:
		for i in j[:-1]:
			if i == j[5]: # Nodelist
				nodes = nodeset.expand(i)
				nodes.sort()
				fout.write('[')
				for n in nodes[:-1]:
					fout.write('{}-'.format(n))
				fout.write('{}],'.format(nodes[-1]))
				continue
			fout.write('{},'.format(i))
		fout.write(int_state_to_str(j[-1]))
		fout.write('\n')
	fout.close()

def training_csv_by_nodes_dump(filename, jobs, nodelist, append=False, skip_header=False):
	""" Works as training_csv_dump but splitting nodelist into one attribute per node """
	fout = open(filename,'a' if append else 'w')
	if not skip_header:
		fout.write("id_job, priority, account, user, nodes_alloc, ")
		
		for n in nodelist:
			fout.write('{}, '.format(n))

		fout.write("cpus_req, cpus_alloc, mem_req, id_qos, partition, "
			"time_start, time_end, timelimit, time_submit, time_eligble, " 
			"time_suspended, time_elapsed, start_week_day, "
			"start_hour, state\n")
	for j in jobs:
		for i in j[:-1]:
			if i == j[5]: # Nodelist
				job_nodes = nodeset.expand(i)
				for n in nodelist:
					if n in job_nodes: fout.write("yes, ")
					else: fout.write("no, ")
				continue
			fout.write('{},'.format(i))
		fout.write(int_state_to_str(j[-1]))
		fout.write('\n')
	fout.close()

