#!/usr/bin/env python
# encoding: utf-8

# Original coder: 	Javier Lopez Cacheiro (CESGA)
# Modified by: 		Alberto M. Esmorís Pena (UOC student)

from __future__ import print_function

import os
import sys

from anomaly.tools import slurm
from anomaly import utils

###  A R G S  ###
#---------------#
started_before_ts = None
started_after_ts = None
end_ts = None
min_etime = 0
out_path = 'exported_metrics.csv'
append = False
skip_header = False
split_nodes = None

def parse_args():
	global started_before_ts, started_after_ts, end_ts
	global min_etime, out_path
	global append, skip_header, split_nodes

	i = 0
	while i < len(sys.argv):
		arg = sys.argv[i]
		if arg == '-h':
			help()
			exit(0)
		if i == 0 :
			i += 1
			continue
		if arg == '-started-before-ts':
			i += 1
			arg = sys.argv[i]
			started_before_ts = arg
		elif arg == '-started-after-ts':
			i += 1
			arg = sys.argv[i]
			started_after_ts = arg
		elif arg == '-end-ts':
			i += 1
			arg = sys.argv[i]
			end_ts = arg
		elif arg == '-min-etime':
			i += 1
			arg = sys.argv[i]
			min_etime = int(arg)
		elif arg == '-out':
			i += 1
			arg = sys.argv[i]
			out_path =  arg
		elif arg == '-append':
			append = True
		elif arg == '-skip-header':
			skip_header = True
		elif arg == '-split-nodes':
			i += 1
			arg = sys.argv[i]
			split_nodes = arg
		else :
			print('ERROR! Unrecognized parameter "%s"' % arg)
			exit(1)
		i += 1

def help():
	print(
'''generate_metrics.py HELP:
	-h
		to print this help
	-out "<path>"
		to specify the path where exported data will be generated
	-started-before-ts <timestamp>
		only jobs which started before this timestamp will be considered
	-started-after-ts <timestamp>
		only jobs which started after this timestamp will be considered
	-end-ts <timestamp>
		to specify the end timestamp considered jobs will have to end before
	-min-etime <seconds>
		to specify the minimum elapsed time that a job needs to be considered
	-append
		when specified, generated metric will be written at the end of the file
		located in the specified output path (by default "exported_metrics.csv"
	-skip-header
		when specified, no header will be printed in generated output
	-split-nodes "<path>"
		specifies the path to a node list file (one node name per line).
		Nodes which appear in this file will be used as yes/no attributes
''')

###  S C R I P T  ###
#-------------------#
parse_args()

if __name__ == '__main__':

	print('Retrieving jobs which started between: {} - {}'.format(started_after_ts, started_before_ts, end_ts))
	jobs = slurm.jobs_mysql(started_before=started_before_ts, started_after=started_after_ts,
		end=end_ts, min_etime=min_etime)
	print('Found {} jobs'.format(len(jobs)))

	if split_nodes is None:
		print('Generating training csv ...')
		utils.training_csv_dump(out_path, jobs, append=append, skip_header=skip_header)
		print('Training csv generated!\n\n')
	else:
		print('Generating training csv splitted by nodes ...')
		utils.training_csv_by_nodes_dump(out_path, jobs, utils.split_nodes_to_list(split_nodes)
				, append=append, skip_header=skip_header)
		print('Training csv splitted by nodes generated!\n\n')
