import sys

# --- CONSTANTS --- #
# ----------------- #
JOBSFILE_ID_INDEX = 0
JOBSFILE_STATE_INDEX = 20
JOBSFILE_ACCURACY_INDEX = 21
JOBSFILE_PREDICTION_INDEX = 22
		
# --- EXTRACT_JOBS --- #
# -------------------- #
def extract_jobs(jobsfile, has_predictions):
	jobs = {}
	line = jobsfile.readline()[:-1]
	while len(line) > 0:
		split = line.split(",")
		if has_predictions:
			jobs.update({split[JOBSFILE_ID_INDEX] :
				split[JOBSFILE_PREDICTION_INDEX]})
		else:
			jobs.update({split[JOBSFILE_ID_INDEX] :
				split[JOBSFILE_STATE_INDEX]})
		line = jobsfile.readline()[:-1]
	return jobs
	

def average_accuracy(notfile):
	n = 0 
	aggregate = 0
	
	line = notfile.readline()[:-1]
	while len(line) > 0:
		split = line.split(",")
		n += 1
		aggregate += float(split[JOBSFILE_ACCURACY_INDEX])
		line = notfile.readline()[:-1]
		
	return aggregate / n

# --- COMPARE --- #
# --------------- #
def compare(notpath, outpath):
	notfile = open(notpath, 'r')
	outfile = open(outpath, 'r')
	
	notfile.readline() # skip header
	jobs_predictions = extract_jobs(notfile, True)
	outfile.readline() # skip header
	jobs_output = extract_jobs(outfile, False)
	
	notfile.close()
	outfile.close()
	
	success_count = 0
	fail_count = 0
	uncovered_count = 0
	
	failed_success_count = 0
	failed_fail_count = 0
	failed_but_completed_count = 0
	failed_but_timeout_count = 0
	
	timeout_success_count = 0
	timeout_fail_count = 0
	timeout_but_completed_count = 0
	timeout_but_failed_count = 0
	
	for prediction_id in jobs_predictions:
		if jobs_output[prediction_id]:
			prediction = jobs_predictions[prediction_id]
			if jobs_output[prediction_id] == prediction:
				success_count += 1
				if prediction == 'FAILED':
					failed_success_count += 1
				elif prediction == 'TIMEOUT':
					timeout_success_count += 1
			else:
				fail_count += 1
				if prediction == 'FAILED':
					failed_fail_count += 1
					if jobs_output[prediction_id] == 'COMPLETED':
						failed_but_completed_count += 1
					elif jobs_output[prediction_id] == 'TIMEOUT':
						failed_but_timeout_count += 1
				elif prediction == 'TIMEOUT':
					timeout_fail_count += 1
					if jobs_output[prediction_id] == 'COMPLETED':
						timeout_but_completed_count += 1
					elif jobs_output[prediction_id] == 'FAILED':
						timeout_but_failed_count += 1
		else:
			uncovered_cases_count += 1
	
	notfile = open(notpath, 'r')
	notfile.readline() # skip_header		
	expected_accuracy = average_accuracy(notfile)
	notfile.close()
			
	print()
	print('Success predictions: {}'.format(success_count))
	print('Failed predictions: {}'.format(fail_count))
	print('Uncovered cases: {}'.format(uncovered_count))
	print('Expected accuracy: {}'.format(expected_accuracy))
	print('Real accuracy: {}'.format(success_count/(success_count+fail_count)))
	print()
	print('FAILED:')
	print('\tSuccess predictions: {}'.format(failed_success_count))
	print('\tFail predictions: {}'.format(failed_fail_count))
	print('\tBut COMPLETED: {}'.format(failed_but_completed_count))
	print('\tBut TIMEOUT: {}'.format(failed_but_timeout_count))
	if failed_success_count > 0 or failed_fail_count > 0 :
		print('\tAccuracy: {}'.format(
		failed_success_count / (failed_success_count + failed_fail_count)))
	print()
	print('TIMEOUT:')
	print('\tSuccess predictions: {}'.format(timeout_success_count))
	print('\tFail predictions: {}'.format(timeout_fail_count))
	print('\tBut COMPLETED: {}'.format(timeout_but_completed_count))
	print('\tBut FAILED: {}'.format(timeout_but_failed_count))
	if timeout_success_count > 0 or timeout_success_count > 0:
		print('\tAccuracy: {}'.format(
			timeout_success_count / (timeout_success_count + timeout_fail_count)))

# --- PARSE ARGS --- #
# ------------------ #
def parse_args(args):
	if len(args) != 3: # args[0] is the script itself
		print('Input MUST have two parameters:\nThe first one, which corresponds '
			'with the notifications file.\nThe second one, which corresponds '
			'with the query file used to analyse predictions.')
		return
	compare(args[1], args[2])

# ----- M A I N ----- #
# ------------------- #
if __name__ == '__main__':
	parse_args(sys.argv)
