package ai.lazuli.cesga;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

import ai.lazuli.cesga.logging.Logger;

public class Config{
	// *** CONSTANTS *** //
	// ***************** //
	// Path
	public static String CONFIG_PATH = "config.cfg";
	
	// Keys
	public static String SHORT_COLLECTION_INTERVAL = "ShortCollectionInterval";
	public static String SHORT_COLLECTION_SCRIPT = "ShortCollectionScript";
	public static String SHORT_DATA_TIME_TO_LIVE = "ShortDataTimeToLive";
	public static String SHORT_DATA_PATH = "ShortDataPath";
	public static String SHORT_MODEL_SCRIPT = "ShortModelScript";
	public static String SHORT_MODEL_PATH = "ShortModelPath";
	
	public static String LONG_DATA_ENABLED = "LongDataEnabled";
	public static String LONG_COLLECTION_SCRIPT = "LongCollectionScript";
	public static String LONG_COLLECTION_INTERVAL = "LongCollectionInterval";
	public static String LONG_DATA_TIME_TO_LIVE = "LongDataTimeToLive";
	public static String LONG_DATA_PATH = "LongDataPath";
	public static String LONG_MODEL_SCRIPT = "LongModelScript";
	public static String LONG_MODEL_PATH = "LongModelPath";
	
	public static String PREDICTOR_INTERVAL = "PredictorInterval";
	public static String PREDICTOR_PATH = "PredictorPath";
	public static String PREDICTOR_PREDICTIONS_TIME_TO_LIVE = "PredictorPredictionsTimeToLive";
	public static String PREDICTOR_COLLECTIONS_TIME_TO_LIVE = "PredictorCollectionsTimeToLive";
	public static String PREDICTOR_SCRIPT_PATH = "PredictorScriptPath";
	public static String PREDICTOR_COLLECTION_PATH = "PredictorCollectionPath";
	public static String PREDICTOR_PREDICTIONS_PATH = "PredictorPredictionsPath";
	public static String PREDICTOR_LAST_COLLECTION_EPOCH = "PredictorLastCollectionEpoch";
	
	public static String NOTIFIER_CONFIGURATION = "NotifierConfiguration";
	public static String NOTIFIER_POOL = "NotifierPool";
	public static String NOTIFICATIONS_PATH = "NotificationsPath";
	
	public static String LOGS_PATH = "LogsPath";
	public static String LOGS_TIME_TO_LIVE = "LogsTimeToLive";
	
	
	// *** ATTRIBUTES *** //
	// ****************** //
	private Properties config;
	private String cfgPath;
	
	// *** SINGLETON *** //
	// ***************** //
	private static Config _config = null;
	
	private Config(String cfgPath) {
		config = new Properties();
		this.cfgPath = cfgPath;
		try {
			File f = new File(this.cfgPath);
			if(f.exists() && f.canRead()) {
				FileInputStream fin = new FileInputStream(f);
				config.load(fin);
				fin.close();
			}
			else {
				File parent = f.getParentFile();
				if(parent!=null) parent.mkdirs();
				f.createNewFile();
				config.setProperty(SHORT_COLLECTION_SCRIPT, "cesga/data/short_collection_script.sh");
				config.setProperty(SHORT_COLLECTION_INTERVAL, "604800"); // in seconds (604800s = 1 week)
				config.setProperty(SHORT_DATA_TIME_TO_LIVE, "1209600"); // in seconds (1209600s = 2 weeks)
				config.setProperty(SHORT_DATA_PATH, "cesga/data/short/collections");
				config.setProperty(SHORT_MODEL_SCRIPT, "cesga/data/c50.R");
				config.setProperty(SHORT_MODEL_PATH, "cesga/data/models/short.rules");
				config.setProperty(LONG_DATA_ENABLED, "true");
				config.setProperty(LONG_COLLECTION_SCRIPT, "cesga/data/long_collection_script.sh");
				config.setProperty(LONG_COLLECTION_INTERVAL, "2419200"); // in seconds (2419200s = 4 weeks)
				config.setProperty(LONG_DATA_TIME_TO_LIVE, "4838400"); // in seconds (5184000s = 8 weeks)
				config.setProperty(LONG_DATA_PATH, "cesga/data/long/collections");
				config.setProperty(LONG_MODEL_SCRIPT, "cesga/data/c50.R");
				config.setProperty(LONG_MODEL_PATH, "cesga/data/models/long.rules");
				config.setProperty(PREDICTOR_INTERVAL, "7200"); // in seconds (7200s = 2 hours)
				config.setProperty(PREDICTOR_PREDICTIONS_TIME_TO_LIVE, "21600"); // in seconds (21600s = 6 hours)
				config.setProperty(PREDICTOR_COLLECTIONS_TIME_TO_LIVE, "14400"); // in seconds (14400s = 4 hours)
				config.setProperty(PREDICTOR_PATH, "cesga/data/predictor"); 
				config.setProperty(PREDICTOR_SCRIPT_PATH, "cesga/data/predictor/data_to_predict_collection_script.sh");
				config.setProperty(PREDICTOR_COLLECTION_PATH, "cesga/data/predictor/collections");
				config.setProperty(PREDICTOR_PREDICTIONS_PATH, "cesga/data/predictor/predictions");
				config.setProperty(PREDICTOR_LAST_COLLECTION_EPOCH, ""+(System.currentTimeMillis()/1000));
				config.setProperty(NOTIFIER_CONFIGURATION, "cesga/data/notifier/config.cfg");
				config.setProperty(NOTIFIER_POOL, "cesga/data/notifier/pool");
				config.setProperty(NOTIFICATIONS_PATH, "cesga/data/notifier/notifications");
				config.setProperty(LOGS_TIME_TO_LIVE, "7776000"); // in seconds (7776000s ~ 3 months)
				config.setProperty(LOGS_PATH, "cesga/logs");
				
				FileOutputStream fout = new FileOutputStream(f);
				config.store(fout, null);
				fout.close();
			}
		}catch(Exception ex) {
			ex.printStackTrace();
		}
	}
	
	public static Config getInstance() {
		if(_config==null)
			_config = new Config(CONFIG_PATH);
		return _config;
	}
	
	public static Config getInstance(String cfgPath) {
		if(_config==null)
			_config = new Config(cfgPath);
		return _config;
	}
	
	// *** GETTERs and SETTERs *** //
	// *************************** //
	public String getShortCollectionScript() {
		return config.getProperty(SHORT_COLLECTION_SCRIPT);
	}
	
	public void setShortCollectionScript(String scriptPath) {
		config.setProperty(SHORT_COLLECTION_SCRIPT, scriptPath);
	}
	
	public long getShortCollectionInterval() {
		return Long.parseLong(config.getProperty(SHORT_COLLECTION_INTERVAL));
	}
	
	public void setShortCollectionInterval(long interval) {
		config.setProperty(SHORT_COLLECTION_INTERVAL, ""+interval);
	}
	
	public long getShortDataTimeToLive() {
		return Long.parseLong(config.getProperty(SHORT_DATA_TIME_TO_LIVE));
	}
	
	public void setShortDataTimeToLive(long ttl) {
		config.setProperty(SHORT_DATA_TIME_TO_LIVE, ""+ttl);
	}
	
	public String getShortDataPath() {
		return config.getProperty(SHORT_DATA_PATH);
	}
	
	public void setShortDataPath(String path) {
		config.setProperty(SHORT_DATA_PATH, path);
	}
	
	public String getShortModelScript() {
		return config.getProperty(SHORT_MODEL_SCRIPT);
	}
	
	public void setShortModelScript(String path) {
		config.setProperty(SHORT_MODEL_SCRIPT, path);
	}
	
	public String getShortModelPath() {
		return config.getProperty(SHORT_MODEL_PATH);
	}
	
	public void setShortModelPath(String path) {
		config.setProperty(SHORT_MODEL_PATH, path);
	}
	
	public boolean isLongDataEnabled() {
		return Boolean.parseBoolean(config.getProperty(LONG_DATA_ENABLED));
	}
	
	public void setLongDataEnabled(boolean enabled) {
		config.setProperty(LONG_DATA_ENABLED, ""+enabled);
	}
	
	public String getLongCollectionScript() {
		return config.getProperty(LONG_COLLECTION_SCRIPT);
	}
	
	public void setLongCollectionScript(String scriptPath) {
		config.setProperty(LONG_COLLECTION_SCRIPT, scriptPath);
	}
	
	public long getLongCollectionInterval() {
		return Long.parseLong(config.getProperty(LONG_COLLECTION_INTERVAL));
	}
	
	public void setLongCollectionInterval(long interval) {
		config.setProperty(LONG_COLLECTION_INTERVAL, ""+interval);
	}
	
	public long getLongDataTimeToLive() {
		return Long.parseLong(config.getProperty(LONG_DATA_TIME_TO_LIVE));
	}
	
	public void setLongDataTimeToLive(long ttl) {
		config.setProperty(LONG_DATA_TIME_TO_LIVE, ""+ttl);
	}
	
	public String getLongDataPath() {
		return config.getProperty(LONG_DATA_PATH);
	}
	
	public void setLongDataPath(String path) {
		config.setProperty(LONG_DATA_PATH, path);
	}
	
	public String getLongModelScript() {
		return config.getProperty(LONG_MODEL_SCRIPT);
	}
	
	public void setLongModelScript(String path) {
		config.setProperty(LONG_MODEL_SCRIPT, path);
	}
	
	public String getLongModelPath() {
		return config.getProperty(LONG_MODEL_PATH);
	}
	
	public void setLongModelPath(String path) {
		config.setProperty(LONG_MODEL_PATH, path);
	}
	
	public long getPredictorInterval() {
		return Long.parseLong(config.getProperty(PREDICTOR_INTERVAL));
	}
	
	public void setPredictorInterval(long interval) {
		config.setProperty(PREDICTOR_INTERVAL, ""+interval);
	}
	
	public long getPredictorPredictionsTimeToLive() {
		return Long.parseLong(config.getProperty(PREDICTOR_PREDICTIONS_TIME_TO_LIVE));
	}
	
	public void setPredictorPredictionsTimeToLive(long ttl) {
		config.setProperty(PREDICTOR_PREDICTIONS_TIME_TO_LIVE, ""+ttl);
	}
	
	public long getPredictorCollectionsTimeToLive() {
		return Long.parseLong(config.getProperty(PREDICTOR_COLLECTIONS_TIME_TO_LIVE));
	}
	
	public void setPredictorCollectionsTimeToLive(long ttl) {
		config.setProperty(PREDICTOR_COLLECTIONS_TIME_TO_LIVE, ""+ttl);
	}
	
	public String getPredictorPath() {
		return config.getProperty(PREDICTOR_PATH);
	}
	
	public void setPredictorPath(String path) {
		config.setProperty(PREDICTOR_PATH, path);
	}
	
	public String getPredictorScriptPath() {
		return config.getProperty(PREDICTOR_SCRIPT_PATH);
	}
	
	public void setPredictorScriptPath(String path) {
		config.setProperty(PREDICTOR_SCRIPT_PATH, path);
	}
	
	public String getPredictorCollectionPath() {
		return config.getProperty(PREDICTOR_COLLECTION_PATH);
	}
	
	public void setPredictorCollectionPath(String path) {
		config.setProperty(PREDICTOR_COLLECTION_PATH, path);
	}
	
	public String getPredictorPredictionsPath() {
		return config.getProperty(PREDICTOR_PREDICTIONS_PATH);
	}
	
	public void setPredictorPredictionsPath(String path) {
		config.setProperty(PREDICTOR_PREDICTIONS_PATH, path);
	}
	
	public long getPredictorLastCollectionEpoch() {
		return Long.parseLong(config.getProperty(PREDICTOR_LAST_COLLECTION_EPOCH));
	}
	
	public void setPredictorLastCollectionEpoch(long epoch) {
		config.setProperty(PREDICTOR_LAST_COLLECTION_EPOCH, ""+epoch);
	}
	
	public long getLogsTimeToLive() {
		return Long.parseLong(config.getProperty(LOGS_TIME_TO_LIVE));
	}
	
	public void setLogsTimeToLive(long ttl) {
		config.setProperty(LOGS_TIME_TO_LIVE, ""+ttl);
	}
	
	public String getNotifierConfiguration() {
		return config.getProperty(NOTIFIER_CONFIGURATION);
	}
	
	public void setNotifierConfiguration(String pathToConfig) {
		config.setProperty(NOTIFIER_CONFIGURATION, pathToConfig);
	}
	
	public String getNotificationsPath() {
		return config.getProperty(NOTIFICATIONS_PATH);
	}
	
	public void setNotificationsPath(String path) {
		config.setProperty(NOTIFICATIONS_PATH, path);
	}
	
	public String getLogsPath() {
		return config.getProperty(LOGS_PATH);
	}
	
	public void setLogsPath(String path) {
		config.setProperty(LOGS_PATH, path);
	}
	
	public String getNotifierPool() {
		return config.getProperty(NOTIFIER_POOL);
	}
	
	public void setNotifierPool(String poolPath) {
		config.setProperty(NOTIFIER_POOL, poolPath);
	}
	
	// *** SAVE *** //
	// ************ //
	public void save() {
		String cfgPath = CONFIG_PATH;
		try {
			FileOutputStream out = new FileOutputStream(cfgPath);
			config.store(out, null);
			out.close();
		}catch(Exception ex) {
			Logger.printf("Could not save config to file: \"%s\"\n", cfgPath);
			Logger.printException(ex);
		}
	}
	
}
