package ai.lazuli.cesga;

import java.util.Arrays;

public class Main {
	public final static byte MODE_HELP = 0;
	public final static byte MODE_START_AI = 1;
	public final static byte MODE_QUERIES = 2;
	
	public static byte mode = MODE_HELP;
	
	// *** M A I N *** //
	// *************** //
	public static void main (String args[]) {
		if(args.length>0) {
			
			String modeStr = args[0];
			
			if(modeStr.equals("-s") || modeStr.equals("--start")) {
				String cfgPath = Config.CONFIG_PATH;
				if(args.length > 1) 
					cfgPath = args[1];
				StartAI.exec(cfgPath);
				return;
			}
			else if(modeStr.equals("-q") || modeStr.equals("--query")) {
				if(args.length < 3) {
					System.err.printf(
						"Query execution requires its first parameter to be the path to the CSV file \n"+
						"and it MUST have at least one statement too.\n");
				}
				else {
					Queries.exec(args[1], Arrays.copyOfRange(args, 2, args.length));
				}
				return;
			}
		
		}
		
		help();
	}
	
	public static void help() {
		System.out.printf(
			"HELP: \n\n"+	
			"This software has three different execution modes, "+
			"which are specified using the first parameter:\n\n"+
			"-h OR --help\n"+
			"\tPrints this help\n"+
			"-s OR --start [CONFIG PATH]\n"+
			"\tStarts the AI\n"+
			"\tMay use the optional last parameter to specify the path to the config file.\n"+
			"\tOtherwise, default config path will be used.\n"+
			"-q OR --query <CSV FILE PATH> <QUERY>\n"+
			"\tPerforms the requested query over the data contained in the CSV file\n"+
			"\tQuery syntax supports the following statements:\n"+
			"\t\tSTATE=[state1,...,stateN]\n"+
			"\t\t\tQueries those jobs which state is contained in the specified list\n"+
			"\t\tUSER=[user1,...,userN]\n"+
			"\t\t\tQueries those jobs which user is contained in the specified list\n"+
			"\t\tACCOUNT=[account1,...,accountN]\n"+
			"\t\t\tQueries those jobs which account is contained in the specified list\n"+
			"\t\tPARTITION=[partition1,...,partitionN]\n"+
			"\t\t\tQueries those jobs which partition is contained in the specified list\n"+
			"\t\tSTART_WEEK_DAY=[weekday1,...,weekdayN]\n"+
			"\t\t\tQueries those jobs which start week day is contained in the specified list\n"+
			"\t\tMIN_ELAPSED_TIME=<Long integer>\n"+
			"\t\t\tQueries those jobs which elapsed time is greater or equal than the specified\n"+
			"\t\tMAX_ELAPSED_TIME=<Long integer>\n"+
			"\t\t\tQueries those jobs which elapsed time is less or equal than the specified\n"+
			"\t\tMIN_NODES_ALLOC=<Long integer>\n"+
			"\t\t\tQueries those jobs which nodes alloc value is greater or equal than the specified\n"+
			"\t\tMAX_NODES_ALLOC=<Long integer>\n"+
			"\t\t\tQueries those jobs which nodes alloc value is less or equal than the specified\n"+
			"\t\tMIN_START_TIME=<Long integer>\n"+
			"\t\t\tQueries those jobs which start time is greater or equal than the specified\n"+
			"\t\tMAX_START_TIME=<Long integer>\n"+
			"\t\t\tQueries those jobs which start time is less or equal than the specified\n"+
			"\t\tMIN_END_TIME=<Long integer>\n"+
			"\t\t\tQueries those jobs which end time is greater or equal than the specified\n"+
			"\t\tMAX_END_TIME=<Long integer>\n"+
			"\t\t\tQueries those jobs which end time is less or equal than the specified\n"+
			"\t\tMIN_CPUS_ALLOC=<Long integer>\n"+
			"\t\t\tQueries those jobs which cpus alloc value is greater or equal than the specified\n"+
			"\t\tMAX_CPUS_ALLOC=<Long integer>\n"+
			"\t\t\tQueries those jobs which cpus alloc value is less or equal than the specified\n"+
			"\t\tMIN_CPUS_REQ=<Long integer>\n"+
			"\t\t\tQueries those jobs which required a number of cpus greater or equal than the specified\n"+
			"\t\tMAX_CPUS_REQ=<Long integer>\n"+
			"\t\t\tQueries those jobs which required a number of cpus less or equal than the specified\n"+
			"\t\tMIN_MEM_REQ=<Long integer>\n"+
			"\t\t\tQueries those jobs which required an amount of memory greater or equal than the specified\n"+
			"\t\tMAX_MEM_REQ=<Long integer>\n"+
			"\t\t\tQueries those jobs which required an amount of memory less or equal than the specified\n"+
			"\t\tNODES=[node1,...,nodeN]\n"+
			"\t\t\tQueries those jobs which were executed at, at least, the specified nodes\n"
		);
	}
}
