package ai.lazuli.cesga;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import ai.lazuli.cesga.data.Querier;
import ai.lazuli.cesga.data.QuerierException;
import ai.lazuli.utils.csv.CSV;
import ai.lazuli.utils.csv.CSVIO;

public class Queries {
	public static void exec(String csvPath, String statements[]) {
		File csvFile = new File(csvPath);
		if(!csvFile.exists()) {
			System.err.printf("Received path \"%s\" denotes a file that does not exist.\n", csvPath);
		}
		
		try {
			FullStatement fs = new FullStatement();
			for(String statement : statements) {
				buildFullStatement(fs, statement);
			}
			CSV csv = CSVIO.read(csvPath, true, null);
			csv = applyFullStatement(csv, fs);
			csv.print(System.out);
		}catch(Exception ex) {
			ex.printStackTrace();
		}
	}
	
	// *** STATEMENTS *** //
	// ****************** //
	public static void buildFullStatement(FullStatement fs, String statement) {
		String stName = statement.substring(0,statement.indexOf("=")).toUpperCase();
		String stValue = statement.substring(1+statement.indexOf("="));
		
		if(stName.equals("STATE"))
			fs.STATE = extractList(stValue);
		else if(stName.equals("USER"))
			fs.USER = extractList(stValue);
		else if(stName.equals("ACCOUNT"))
			fs.ACCOUNT = extractList(stValue);
		else if(stName.equals("PARTITION"))
			fs.PARTITION = extractList(stValue);
		else if(stName.equals("START_WEEK_DAY"))
			fs.START_WEEK_DAY = extractList(stValue);
		else if(stName.equals("MIN_ELAPSED_TIME"))
			fs.MIN_ELAPSED_TIME = Long.parseLong(stValue);
		else if(stName.equals("MAX_ELAPSED_TIME"))
			fs.MAX_ELAPSED_TIME = Long.parseLong(stValue);
		else if(stName.equals("MIN_NODES_ALLOC"))
			fs.MIN_NODES_ALLOC = Long.parseLong(stValue);
		else if(stName.equals("MAX_NODES_ALLOC"))
			fs.MAX_NODES_ALLOC = Long.parseLong(stValue);
		else if(stName.equals("MIN_START_TIME"))
			fs.MIN_START_TIME = Long.parseLong(stValue);
		else if(stName.equals("MAX_START_TIME"))
			fs.MAX_START_TIME = Long.parseLong(stValue);
		else if(stName.equals("MIN_END_TIME"))
			fs.MIN_END_TIME = Long.parseLong(stValue);
		else if(stName.equals("MAX_END_TIME"))
			fs.MAX_END_TIME = Long.parseLong(stValue);
		else if(stName.equals("MIN_CPUS_ALLOC"))
			fs.MIN_CPUS_ALLOC = Long.parseLong(stValue);
		else if(stName.equals("MAX_CPUS_ALLOC"))
			fs.MAX_CPUS_ALLOC = Long.parseLong(stValue);
		else if(stName.equals("MIN_CPUS_REQ"))
			fs.MIN_CPUS_REQ = Long.parseLong(stValue);
		else if(stName.equals("MAX_CPUS_REQ"))
			fs.MAX_CPUS_REQ = Long.parseLong(stValue);
		else if(stName.equals("MIN_MEM_REQ"))
			fs.MIN_MEM_REQ = Long.parseLong(stValue);
		else if(stName.equals("MAX_MEM_REQ"))
			fs.MAX_MEM_REQ = Long.parseLong(stValue);
		else if(stName.equals("NODES"))
			fs.NODELIST = extractList(stValue);
	}
	
	public static CSV applyFullStatement(CSV input, FullStatement fs) throws QuerierException{
		CSV output = input; 
		
		if(fs.STATE != null)
			output = Querier.getJobsByState(output, fs.STATE);
		if(fs.USER != null)
			output = Querier.getJobsByUser(output, fs.USER);
		if(fs.ACCOUNT != null)
			output = Querier.getJobsByAccount(output, fs.ACCOUNT);
		if(fs.PARTITION != null)
			output = Querier.getJobsByPartition(output, fs.PARTITION);
		if(fs.START_WEEK_DAY != null)
			output = Querier.getJobsByStartWeekDay(output, fs.START_WEEK_DAY);
		if(fs.MIN_ELAPSED_TIME != null || fs.MAX_ELAPSED_TIME != null)
			output = Querier.getJobsByElapsedTime(output,
					fs.MIN_ELAPSED_TIME, fs.MAX_ELAPSED_TIME);
		if(fs.MIN_NODES_ALLOC != null || fs.MAX_NODES_ALLOC != null)
			output = Querier.getJobsByNodesAlloc(output,
					fs.MIN_NODES_ALLOC, fs.MAX_NODES_ALLOC);
		if(fs.MIN_START_TIME != null || fs.MAX_START_TIME != null)
			output = Querier.getJobsByStartTime(output,
					fs.MIN_START_TIME, fs.MAX_START_TIME);
		if(fs.MIN_END_TIME != null || fs.MAX_END_TIME != null)
			output = Querier.getJobsByStartTime(output,
					fs.MIN_END_TIME, fs.MAX_END_TIME);
		if(fs.MIN_CPUS_ALLOC != null || fs.MAX_CPUS_ALLOC != null)
			output = Querier.getJobsByCpusAlloc(output,
					fs.MIN_CPUS_ALLOC, fs.MAX_CPUS_ALLOC);
		if(fs.MIN_CPUS_REQ != null || fs.MAX_CPUS_REQ != null)
			output = Querier.getJobsByCpusReq(output,
					fs.MIN_CPUS_REQ, fs.MAX_CPUS_REQ);
		if(fs.MIN_MEM_REQ != null || fs.MAX_MEM_REQ != null)
			output = Querier.getJobsByCpusReq(output,
					fs.MIN_MEM_REQ, fs.MAX_MEM_REQ);
		if(fs.NODELIST != null)
			output = Querier.getJobsByNodelist(output, fs.NODELIST);
		
		return output;
	}
	
	// *** AUXILIAR FUNCTIONS *** //
	// ************************** //
	public static List<String> extractList(String stValue){
		int indexOfOpenList = -1;
		int indexOfCloseList = -1;
		for(int i = 0 ; i < stValue.length() ; i++) {
			char c = stValue.charAt(i);
			if(c=='[' && indexOfOpenList == -1) {
				indexOfOpenList = i;
			}
			if(c==']') indexOfCloseList = i;
		}
		
		String csvValues = stValue.substring(indexOfOpenList+1, indexOfCloseList);
		return new ArrayList<String>(Arrays.asList(csvValues.split(",")));
	}
}

class FullStatement{
	public List<String> STATE = null;
	public List<String> USER = null;
	public List<String> ACCOUNT = null;
	public List<String> PARTITION = null;
	public List<String> START_WEEK_DAY = null;
	public Long MIN_ELAPSED_TIME = null;
	public Long MAX_ELAPSED_TIME = null;
	public Long MIN_NODES_ALLOC = null;
	public Long MAX_NODES_ALLOC = null;
	public Long MIN_START_TIME = null;
	public Long MAX_START_TIME = null;
	public Long MIN_END_TIME = null;
	public Long MAX_END_TIME = null;
	public Long MIN_CPUS_ALLOC = null;
	public Long MAX_CPUS_ALLOC = null;
	public Long MIN_CPUS_REQ = null;
	public Long MAX_CPUS_REQ = null;
	public Long MIN_MEM_REQ = null;
	public Long MAX_MEM_REQ = null;
	public List<String> NODELIST = null;
}

