package ai.lazuli.cesga.data;

public class ModelSynchronizer {
	// *** ATTRIBUTES *** //
	// ****************** //
	private Integer readOperations;
	private Boolean writeMutex;
	
	// *** CONSTRUCTOR *** //
	// ******************* //
	public ModelSynchronizer() {
		this.readOperations = 0;
		this.writeMutex = false;
	}
	
	// *** METHODS *** //
	// *************** //
	/**
	 * Try to obtain safe synchronized read access to model rules.
	 * After this access was obtained and desired operations were performed,
	 * it is a MUST to release, to avoid deadlocks.
	 * @return TRUE when read access to model file can be
	 * safely performed. FALSE otherwise.
	 */
	public boolean tryModelReadOperation() {
		boolean output = false;
		synchronized(writeMutex) {
			if(!writeMutex) {
				synchronized(readOperations) {
					readOperations++;
				}
				output = true;
			}
		}
		return output;
	}
	
	/**
	 * Release requested read access to model rules.
	 */
	public void releaseModelReadOperation() {
		synchronized(readOperations) {
			if(readOperations > 0)
				readOperations--;
		}
	}
	
	/**
	 * Try to lock model to perform writting operations.
	 * The mutex must be released after performing desired operations,
	 * otherwise deadlocks may occur.
	 * @return TRUE when write access to model file can be
	 * safely performed. FALSE otherwise.
	 */
	public boolean lockWriteMutex() {
		boolean output = false;
		synchronized (readOperations) {
			if(readOperations == 0) {
				synchronized (writeMutex) {
					if(!writeMutex) {
						writeMutex = true;
						output = true;
					}
				}
				
			}
		}
		return output;
	}
	
	/**
	 * Release requested write access to model rules.
	 */
	public void unlockWriteMutex() {
		synchronized(writeMutex) {
			writeMutex = false;
		}
	}
	
}
