package ai.lazuli.cesga.data;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import ai.lazuli.cesga.logging.Logger;
import ai.lazuli.utils.files.FilePurger;

/**
 * Purges old classification files
 * @author Alberto M. Esmorís Pena
 */
public class Purger extends Thread{
	// *** ATTRIBUTES *** //
	// ****************** //
	private List<File> dirs; // Directories to purge
	private boolean purging;
	private long purgingInterval; // How many time (seconds) between purging operations
	private long maxOld; // Files older than maxOld should be purged
	
	// *** CONSTRUCTORS *** //
	// ******************** //
	public Purger(long purgingInterval, long maxOld, File ... dirs ) {
		setPurging(false);
		setMaxOld(maxOld);
		setDirs(new ArrayList<File>(Arrays.asList(dirs)));
		setPurgingInterval(purgingInterval);
	}
	
	// *** GETTERs and SETTErs *** //
	// *************************** //
	public List<File> getDirs() {
		return dirs;
	}


	public void setDirs(List<File> dirs) {
		this.dirs = dirs;
	}


	public boolean isPurging() {
		return purging;
	}

	public void setPurging(boolean purging) {
		this.purging = purging;
	}

	public long getPurgingInterval() {
		return purgingInterval;
	}


	public void setPurgingInterval(long purgingInterval) {
		this.purgingInterval = purgingInterval;
	}
	
	public long getMaxOld() {
		return maxOld;
	}

	public void setMaxOld(long maxOld) {
		this.maxOld = maxOld;
	}

	// *** START / STOP *** //
	// ******************** //
	public void startPurging() {
		setPurging(true);
		start();
	}
	
	// *** RUN *** //
	// *********** //
	@Override
	public void run() {
		while(purging) {
			for(File dir : getDirs()) {
				try {
					List<File> purgedFiles = FilePurger.purgeByModificationDate(dir, getMaxOld());
					if(purgedFiles.size()>0) {
						String pfStr = "";
						for(File pf : purgedFiles)
							pfStr += String.format("\t'%s' was DELETED\n", pf.getAbsolutePath());
						Logger.printf("PURGED FILES:\n%s", pfStr);
					}
				}catch(IOException ioex) {
					Logger.printException(ioex);
				}
			}
			try {
				sleep(getPurgingInterval()*1000);
			}catch(InterruptedException iex) {
				Logger.printException(iex);
			}
		}
	}
}
