package ai.lazuli.cesga.data;

import java.util.List;

import ai.lazuli.cesga.data.collection.PredictAfterCollection;
import ai.lazuli.utils.csv.CSV;
import ai.lazuli.utils.csv.CSVField;

public class Querier {
	// *** CONSTANTS *** //
	// ***************** //
	public final static String FIELD_STATE = "state";
	public final static String FIELD_USER = "user";
	public final static String FIELD_ACCOUNT = "account";
	public final static String FIELD_MEM_REQ = "mem_req";
	public final static String FIELD_CPUS_REQ = "cpus_req";
	public final static String FIELD_CPUS_ALLOC = "cpus_alloc";
	public final static String FIELD_END_TIME = "time_end";
	public final static String FIELD_START_TIME = "time_start";
	public final static String FIELD_NODES_ALLOC = "nodes_alloc";
	public final static String FIELD_ELAPSED_TIME = "time_elapsed";
	public final static String FIELD_START_WEEK_DAY = "start_week_day";
	public final static String FIELD_PARTITION = "partition";
	public final static String FIELD_NODELIST = "nodelist";
	
	// *** QUERIES *** //
	// *************** //
	public static CSV getJobsByState(CSV input, List<String> states) throws QuerierException{
		return queryValueInList(input, states, FIELD_STATE);
	}
	
	public static CSV getJobsByPredictedState(CSV input, List<String> states) throws QuerierException{
		return queryValueInList(input, states, PredictAfterCollection.FIELD_PREDICTED_CLASS);
	}
	
	public static CSV getJobsByUser(CSV input, List<String> users) throws QuerierException{
		return queryValueInList(input, users, FIELD_USER);
	}
	
	public static CSV getJobsByAccount(CSV input, List<String> accounts) throws QuerierException{
		return queryValueInList(input, accounts, FIELD_ACCOUNT);
	}
	
	public static CSV getJobsByPartition(CSV input, List<String> partitions) throws QuerierException{
		return queryValueInList(input, partitions, FIELD_PARTITION);
	}
	
	public static CSV getJobsByStartWeekDay(CSV input, List<String> startDays) throws QuerierException{
		return queryValueInList(input, startDays, FIELD_START_WEEK_DAY);
	}
	
	public static CSV getJobsByElapsedTime(CSV input, Long min, Long max) throws QuerierException{
		return queryByLongRange(input, min, max, FIELD_ELAPSED_TIME);
	}
	
	public static CSV getJobsByNodesAlloc(CSV input, Long min, Long max) throws QuerierException{
		return queryByLongRange(input, min, max, FIELD_NODES_ALLOC);
	}

	public static CSV getJobsByStartTime(CSV input, Long min, Long max) throws QuerierException{
		return queryByLongRange(input, min, max, FIELD_START_TIME);
	}
	
	public static CSV getJobsByEndTime(CSV input, Long min, Long max) throws QuerierException{
		return queryByLongRange(input, min, max, FIELD_END_TIME);
	}
	
	public static CSV getJobsByCpusAlloc(CSV input, Long min, Long max) throws QuerierException{
		return queryByLongRange(input, min, max, FIELD_CPUS_ALLOC);
	}	
	
	public static CSV getJobsByCpusReq(CSV input, Long min, Long max) throws QuerierException{
		return queryByLongRange(input, min, max, FIELD_CPUS_REQ);
	}	
	
	public static CSV getJobsByMemReq(CSV input, Long min, Long max) throws QuerierException{
		return queryByLongRange(input, min, max, FIELD_MEM_REQ);
	}	
	
	public static CSV getJobsByNodelist(CSV input, List<String> nodes) throws QuerierException{
		try {
			List<CSVField> header = input.getHeader();
			CSV output = new CSV(header);
			
			for(int i = 0 ; i < input.size() ; i++) {
				String val = (String) input.getValue(i, FIELD_NODELIST);
				for(String node : nodes) {
					if(val.contains(node)) {
						output.addRecord(input.getRecord(i));
						continue;
					}
				}
			}
			
			return output;
		}catch(Exception ex) {
			throw new QuerierException("Exception occurred when querying jobs by nodelist",ex);
		}
	}
	
	// *** INNER FUNCTIONS *** //
	// *********************** //
	private static CSV queryByLongRange(CSV input, Long min, Long max, String fieldName)
			throws QuerierException{
		try {
			List<CSVField> header = input.getHeader();
			CSV output = new CSV(header);
			
			for(int i = 0 ; i < input.size() ; i++) {
				Long val = (Long) input.getValue(i, fieldName);
				if(val==null) {
					throw new QuerierException(String.format(
							"No field '%s' was found in input data.",
							fieldName));
				}
				if(	(min==null || val >= min) && (max==null || val <= max) )
					output.addRecord(input.getRecord(i));
			}
			
			return output;
		}catch(Exception ex) {
			throw new QuerierException(String.format(
						"Exception occurred when querying jobs by '%s'", 
						fieldName),
					ex);
		}
	}
	
	private static CSV queryValueInList(CSV input, List<String> list, String fieldName) 
			throws QuerierException{
		try {
			List<CSVField> header = input.getHeader();
			CSV output = new CSV(header);
			
			for(int i = 0 ; i < input.size() ; i++) {
				String val = (String) input.getValue(i, fieldName);
				if(val==null) {
					throw new QuerierException(String.format(
							"No field '%s' was found in input data.",
							fieldName));
				}
				if(list.contains(val))
					output.addRecord(input.getRecord(i));
			}
			
			return output;
		}catch(Exception ex) {
			throw new QuerierException(String.format(
					"Exception occurred when querying jobs by '%s'", 
					fieldName),
				ex);
		}
	}
	
}
