package ai.lazuli.cesga.logging;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;

public class Logger {
	// *** CONSTANTS *** //
	// ***************** //
	public static String LOGS_DIR = "LOGS";
	
	// *** LOG METHODS *** //
	// ******************* //
	public static void print(String str) {
		File dir = new File(LOGS_DIR);
		if(!dir.exists()) dir.mkdirs();
		
		Calendar cal = Calendar.getInstance();
		File subDir = new File(dir.getAbsolutePath()+
				File.separator+cal.get(Calendar.YEAR)+
				File.separator+(cal.get(Calendar.MONTH)+1));
		
		if(!subDir.exists()) subDir.mkdirs();
		
		File logFile = new File(String.format(
				"%s%sDAY_%d.log",
				subDir.getAbsolutePath(),
				File.separator,
				cal.get(Calendar.DAY_OF_MONTH)));
		
		FileOutputStream out = null;
		try {
			out = new FileOutputStream(logFile, true);
			out.write(
					String.format(
						"[%02d:%02d:%02d] %s",
						cal.get(Calendar.HOUR_OF_DAY),
						cal.get(Calendar.MINUTE),
						cal.get(Calendar.SECOND),
						str
					).getBytes());
		}catch(Exception ex) {
			ex.printStackTrace();
		}finally {
			try {
				out.close();
			}catch(IOException ioex) {
				ioex.printStackTrace();
			}
		}
		
	}
	
	public static void println(String str) {
		print(str+"\n");
	}
	
	public static void printf(String format, Object ... args) {
		print(String.format(format, args));
	}
	
	public static void printException(Exception ex) {
		String str = ex.toString()+"\n"+"\t"+ex.getMessage()+"\n";
		for(StackTraceElement e : ex.getStackTrace())
			str += String.format(
				"\t\tFILE %s : %s.%s at LINE %d\n",
				e.getFileName(),
				e.getClassName(),
				e.getMethodName(),
				e.getLineNumber());
		print(str);
	}
}
