package ai.lazuli.utils.classificationrules;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

public class ClassificationModel {
	// *** ATTRIBUTES *** //
	// ****************** //
	private String modelId;
	private List<ClassificationRules> ruleSets;
	
	// *** CONSTRUCTORS *** //
	// ******************** //
	public ClassificationModel(String modelId, List<ClassificationRules> ruleSets) {
		super();
		this.modelId = modelId;
		this.ruleSets = ruleSets;
	}
	
	// *** GETTERs and SETTERs *** //
	// *************************** //
	public String getModelId() {
		return modelId;
	}
	
	public void setModelId(String modelId) {
		this.modelId = modelId;
	}
	public List<ClassificationRules> getRuleSets() {
		return ruleSets;
	}
	public void setRuleSets(List<ClassificationRules> ruleSets) {
		this.ruleSets = ruleSets;
	}
	
	// *** TO STRING *** //
	// ***************** //
	@Override
	public String toString() {
		String str = String.format("Classificaiton model: \"%s\"\n\n",
				getModelId());
		
		for(ClassificationRules cr : ruleSets) {
			str += cr.toString();
		}
		
		return str;
	}
	
	// *** METHODS *** //
	// *************** //
	public Classification classify(Hashtable<String, Object> set) 
			throws ClassificationRuleException {
		List<Classification> classifications = new ArrayList<>();
		Hashtable<String, Integer> votes = new Hashtable<>();
		for(ClassificationRules ruleset : ruleSets) {
			Classification cl = ruleset.classify(set);
			classifications.add(cl);
			if(votes.containsKey(cl.getClassName())) {
				votes.put(cl.getClassName(),
						1+votes.get(cl.getClassName()));
			}else {
				votes.put(cl.getClassName(), 1);
			}
		}
		
		Classification classification = classifications.get(0);
		int cVotes = votes.get(classification.getClassName());
		for(int i = 1 ; i < classifications.size() ; i++) {
			Classification cl = classifications.get(i);
			int v = votes.get(cl.getClassName());
			if(v > cVotes) classification = cl;
		}
		
		return classification;
	}
}
