package ai.lazuli.utils.classificationrules;

import java.util.Hashtable;
import java.util.List;
import java.util.Locale;

public class ClassificationRule {
	// *** CONSTANTS *** //
	// ***************** //		
	
	// *** ATTRIBUTES *** //
	// ****************** //
	private String id;
	private int coveredCases;
	private int okCases;
	private double lift;
	private List<ClassificationRuleCondition> conditions;
	private Classification classification;
	
	
	// *** CONSTRUCTOR *** //
	// ******************* //
	public ClassificationRule(String id, int coveredCases, int okCases, double lift,
			List<ClassificationRuleCondition> conditions, Classification classification) {
		setId(id);
		setCoveredCases(coveredCases);
		setOkCases(okCases);
		setLift(lift);
		setConditions(conditions);
		setClassification(classification);
	}

	// *** GETTERs and SETTERs *** //
	// *************************** //
	public String getId() {
		return id;
	}


	public void setId(String id) {
		this.id = id;
	}

	public int getCoveredCases() {
		return coveredCases;
	}

	public void setCoveredCases(int coveredCases) {
		this.coveredCases = coveredCases;
	}

	public int getOkCases() {
		return okCases;
	}

	public void setOkCases(int okCases) {
		this.okCases = okCases;
	}
	
	public int getFailedCases() {
		return getCoveredCases()-getOkCases();
	}

	public double getLift() {
		return lift;
	}


	public void setLift(double lift) {
		this.lift = lift;
	}


	public List<ClassificationRuleCondition> getConditions() {
		return conditions;
	}


	public void setConditions(List<ClassificationRuleCondition> conditions) {
		this.conditions = conditions;
	}


	public Classification getClassification() {
		return classification;
	}


	public void setClassification(Classification classification) {
		this.classification = classification;
	}


	// *** TO STRING *** //
	// ***************** //
	@Override
	public String toString() {
		String str = String.format(Locale.ROOT,
				"Rule %s: (cover=%d , ok=%d , failed=%d , lift=%f)\n",
				getId(),
				getCoveredCases(),
				getOkCases(),
				getFailedCases(),
				getLift());
	
		for(ClassificationRuleCondition cond : getConditions()) 
			str += cond.toString();
		
		str += String.format(Locale.ROOT,
				"\t-> class %s [%f]\n", getClassification().getClassName(),
				getClassification().getAccuracy());
		
		return str;
	}

	// *** METHODS *** //
	// *************** //
	/**
	 * @return TRUE if this rule is satisfied by the attributes in the received hashtable.
	 * FALSE otherwise.
	 */
	public boolean isSatisfiedBy(Hashtable<String, Object> set) throws ClassificationRuleException{
		Boolean satisfied = null;
		
		for(ClassificationRuleCondition cond : conditions) {
			if(satisfied==null)
				satisfied = cond.isSatisfiedBy(set);
			else
				satisfied &= cond.isSatisfiedBy(set);
		
			if(!satisfied) break;
		}
		
		if(satisfied==null)
			throw new ClassificationRuleException("Rule satisfaction could not be checked.");
		return satisfied;
	}
	
	
}
